package se.umu.jobsubmission.impl.informationfinder; 

import java.util.Calendar;

import org.apache.log4j.Logger;
import org.globus.mds.glue.GLUECERPType;

import se.umu.jobsubmission.util.Worker;

// Worker class for running ResourceFinders...
public class ConverterWorker extends Worker {

	private String converterClasses[]; 
    private Object resourceKey; 
    private ResourceInformationCache resourceInformationCache; 
    
    private static Logger logger = Logger.getLogger(ConverterWorker.class); 

    public ConverterWorker(Object workUnit, 
    		String converters[], 
			Object resourceKey, 
			ResourceInformationCache resourceInformationCache) {
        super(workUnit); 
        this.converterClasses = converters; 
        this.resourceKey = resourceKey; 
        this.resourceInformationCache = resourceInformationCache; 
    }

    public void doWork() {
        // workUnit hold the resource information in the native format
    	logger.debug("Converterworker, converting: " + workUnit); 
    	
    	/*
    	if(workUnit instanceof GLUECERPType) {
    		logger.debug("Using cached, already converted information -> done"); 
    		resultSet.add(workUnit); 
    		return;
    	}*/ 
    	
    	// Try each converter type in order of appearance. 
    	// Should one succeed, skip the rest. 
    	// If all fail, ignore this resource. 
    	Object information = null; 
    	boolean done = false; 
    	for(int i = 0; i < converterClasses.length && !done; i++) {
    		try {
    			logger.debug("trying to convert description using: " 
    					+ converterClasses[i]); 
    			InformationConverter converter; 
    			converter = (InformationConverter) Class.forName(
    					converterClasses[i]).newInstance(); 
				information = converter.convertInformation(workUnit, 
						this.resourceInformationCache); 
				done = true; 
    		} catch (Exception e) {
    			logger.info("Error converting using: " + converterClasses[i]); 
    			information = null; 
    		} 
    	}
    	
    	if(information == null) {
    		logger.error("Could not convert " + workUnit); 
    		return; 
    	}
    	
    	// Converter read result from cache, and returns the result
    	// instead of a new cache entry... 
    	if(information instanceof GLUECERPType) {
    		logger.info("Using already cached information"); 
    		resultSet.add(information); 
    	}
    	else {
        
    		CachedResourceInformation cachedInfo = 
    			(CachedResourceInformation) information; 
    		
    		resultSet.add(cachedInfo.getInformation());         
    		
    		// Add converted information to resource cache. 
    		logger.debug("Adding info for resource: " + resourceKey + " to cache"); 
    		Calendar now = Calendar.getInstance(); 
    		Calendar timeStamp = cachedInfo.getTimestamp(); 
    		
    		long nowS = now.getTimeInMillis(); 
    		long thenS = timeStamp.getTimeInMillis(); 
    		logger.debug("Info is valid to " + timeStamp.getTime() + 
    				" that is, for "  + (thenS - nowS) + " ms"); 
    		
    		this.resourceInformationCache.put(resourceKey, 
    				cachedInfo.getInformation(), 
					cachedInfo.getTimestamp()); 
        
    	}
  
    }
}

