/*
 * Created on Apr 22, 2005
 *
 * Client for committing reservations - test purposes only
 *
 */
package se.umu.wsag.client;

import java.io.FileInputStream;
import java.util.Calendar;

import javax.xml.rpc.Stub;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.ggf.ws_agreement.Agreement;
import org.ggf.ws_agreement.AgreementServiceAddressingLocator;
import org.ggf.ws_agreement.TermTreeType;
import org.globus.axis.util.Util;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.oasis.wsrf.lifetime.SetTerminationTime;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.xml.sax.InputSource;

import se.umu.wsag.impl.AgreementConstants;
import se.umu.reservation.ReservationType;


public class CommitClient {

	static {
		Util.registerTransport(); 
	}

	public static void main(String[] args) {

		try {
        if(args.length < 1) {
            System.err.println("Usage: ReservationCancel eprFile");
            System.exit(1); 
        }

        AgreementServiceAddressingLocator locator = 
        	new AgreementServiceAddressingLocator(); 
        //ReservationServiceAddressingLocator locator = 
          //  new ReservationServiceAddressingLocator(); 
        
        String eprFileName = args[0]; 

        // Get EPR from file
        FileInputStream fileInputStream = new FileInputStream(eprFileName); 
        EndpointReferenceType epr = (EndpointReferenceType)
            ObjectDeserializer.deserialize( 
                    new InputSource(fileInputStream), 
                    EndpointReferenceType.class); 
        fileInputStream.close();
                
        Agreement agreement = locator.getAgreementPort(epr);
        
   		// Client security descriptor file     		
        String clientDesc = "etc/wsag/client-security-config.xml"; 
        // Set descriptor on Stub             
        ((Stub)agreement)._setProperty(
        		Constants.CLIENT_DESCRIPTOR_FILE, 
				clientDesc);    	
        

        // Get AgreementTerms   
        GetResourcePropertyResponse termsResponse = 
        	agreement.getResourceProperty(AgreementConstants.TERMS_RP);
        MessageElement any[] = termsResponse.get_any();
        any = termsResponse.get_any();
        
        // 1: Extract TermTree        
        TermTreeType terms = (TermTreeType) ObjectDeserializer.toObject(
        		any[0], TermTreeType.class); 
                
        any = terms.getAll().getServiceDescriptionTerm().get_any();         
        ReservationType reservationInfo = (ReservationType) ObjectDeserializer.
			toObject(any[0], ReservationType.class); 
  
        System.out.println("Reservation:"); 
        System.out.println(reservationInfo.getLocalID()); 
        
        Calendar startTime = reservationInfo.getExactStart(); 
        int duration = reservationInfo.getDuration(); 
        
        System.out.println("StartTime: " + startTime.getTime());
        System.out.println("Duration: " + duration);

        
		// Add an extra minute just in case...
        startTime.add(Calendar.SECOND, duration + 60);
        System.out.println("Terminating agreement at: " + startTime.getTime());
        
        SetTerminationTime request = new SetTerminationTime(startTime);         
        agreement.setTerminationTime(request); 
                            
        
		} catch (Exception e) {
			System.err.println("Error!"); 
			e.printStackTrace(); 
		}
	}
	
}
